# -*- coding: utf-8 -*-
import os
os.environ['OPENBLAS_NUM_THREADS'] = '1'
import numpy as np
import pandas as pd
import cPickle as pickle
from scipy.interpolate import RectBivariateSpline
from scipy.optimize import root
from scipy.optimize import brentq
import approximate
import simulate_MPI as simulate
import mkl
import getpass
from scipy import stats
mkl.set_num_threads(1)
print(simulate.rank)
import sys
import gc
"""
Default Parameters
"""
calibrationfile='calibrations/calibration_data/baseline_calibration_revision.csv'
num_k = 2500




############################
# Load calibration         #
############################
calibration=pd.read_csv(calibrationfile).T
calibration.columns=calibration.iloc[0,:]
calibration=calibration.iloc[1:,:]


# shock process calibration
psi = calibration.psi.val
ineq = calibration.ineq_flag.val
rho_MU=calibration.rho_MU.val
rho_Theta=calibration.rho_Theta.val # only used in tfpar1 case
sigma_eps=calibration.sigma_eps.val
sigma_epsMU = calibration.sigma_epsMU.val
sigma_Eps_theta_ar = calibration.sigma_Eps_theta_ar.val
sigma_e = calibration.sigma_e.val
sigma_eta= calibration.sigma_eta.val
rho_e=calibration.rho_e.val
Gov = calibration.Gov.val
epsilon = calibration.epsilonbar.val
sigma = calibration.ra.val
gamma = calibration.gamma.val
beta = calibration.beta.val
Bbar = calibration.Bbar.val
onlyineq = 0.
ar = 1.
divMPC = 0.
YSS = None
no_idio = 0.



"""
Import script parameters
"""
argv = sys.argv
if len(argv) < 3:
    print "Need at least 3 arguments"
    exit(1)

runtype = argv[1]
fiscaltype = argv[2]

print ("runtype = "+runtype)
print ("fiscaltype = "+fiscaltype)

## LOAD MODULE

if runtype in ["benchmark","benchmark_highlabor","benchmark_highdiv","benchmark_highbond","benchmark_util","benchmark_lowlabor","benchmark_lowdiv","benchmark_hetexposure"]:
    if fiscaltype == "fiscal":
        import calibrations.calibration_begs2_benchmark_fiscal_decomp as Para
    elif fiscaltype == "nofiscal":
        import calibrations.calibration_begs2_benchmark_nofiscal_decomp as Para
    else:
        print "Second Argument Needs to be fiscal or nofiscal"
        exit(1)
       
elif runtype == "mutual":
    import calibrations.calibration_begs2_mutual_nofiscal_decomp as Para
    
elif runtype == "htm":
    import approximate_NSZ as approximate
    import calibrations.calibration_begs2_revision_htm_nofiscal_decomp_NSZ as Para
    
else:
    print "First Argument Needs to be benchmark or stationary or htm or pareto"
    exit(1)



recession_data=pd.read_excel('calibrations/calibration_data/recession_earnings_loss.xlsx')

recession_data=recession_data[(recession_data.percentile>0)&(recession_data.percentile<100)]
recession_data.set_index('percentile',inplace=True)

#scale correctly for near permanent shocks
ineq_f1=calibration.ineq_f1.val
ineq_f2=calibration.ineq_f2.val
ineq_f3=calibration.ineq_f3.val
ineq_scaling=calibration.ineq_scaling.val
polyE = np.polyfit(stats.norm.ppf([0.05,.5,.95])*ineq_scaling,recession_data.avg[[5,50,95]]-[ineq_f1,ineq_f2,ineq_f3],2)   #.9 scales the standard deviation of e
#Para.E = np.poly1d(polyE)

fE = lambda x: np.exp(2*x)/(1+np.exp(2*x))
polyE2 = np.polyfit(fE(stats.norm.ppf([0.05,.5,.95])*ineq_scaling),recession_data.avg[[5,50,95]]-[ineq_f1,ineq_f2,ineq_f3],2)
Para.E = lambda x : np.poly1d(polyE2)(fE(x))



## LOAD INITIAL CONDITIONS
filetype =""
if runtype in ["benchmark","benchmark_highlabor","benchmark_highdiv","benchmark_highbond","benchmark_hetexposure"]:
    print ("setting initial conditions for " + filetype)
    if runtype =="benchmark" or runtype == "benchmark_hetexposure":
        with open('InitialConditions/data_benchmark.pickle', 'r') as f:
            log_w,Gov,sigma,gamma,beta,c,l,b,omega,T,Tau_w,Tau_d,Tau_b,s,epsilon,mu = pickle.load( f)
    if runtype == "benchmark_highlabor":
        with open('InitialConditions/data_benchmark_highlabor.pickle', 'r') as f:
            log_w,Gov,sigma,gamma,beta,c,l,b,omega,T,Tau_w,Tau_d,Tau_b,s,epsilon,mu = pickle.load( f)
    if runtype == "benchmark_highdiv":
        with open('InitialConditions/data_benchmark_highdiv.pickle', 'r') as f:
            log_w,Gov,sigma,gamma,beta,c,l,b,omega,T,Tau_w,Tau_d,Tau_b,s,epsilon,mu = pickle.load( f)
    if runtype == "benchmark_highbond":
        with open('InitialConditions/data_benchmark_highbond.pickle', 'r') as f:
            log_w,Gov,sigma,gamma,beta,c,l,b,omega,T,Tau_w,Tau_d,Tau_b,s,epsilon,mu = pickle.load( f)

    e = log_w
    e -= e.mean()
    s_vec = (1-Tau_d)*s + Tau_d*1
    
    N = len(e)
    uc = c**(-sigma)
    
    m = 1./uc
    logm = np.log(m)
    logm -= np.mean(logm)
    muhat = mu/np.exp(logm)
    
    if fiscaltype=="no fiscal":
        Gamma = np.zeros((N,6))
        Gamma[:,0] = logm
        Gamma[:,1] = logm
        Gamma[:,2] = muhat
        Gamma[:,3] = omega
        Gamma[:,4] = e
        Gamma[:,5] = s_vec
    else:
        Gamma = np.zeros((N,5))
        Gamma[:,0] = logm
        Gamma[:,1] = muhat
        Gamma[:,2] = omega
        Gamma[:,3] = e
        Gamma[:,4] = s_vec
    Para.TauTarget = Tau_w
    

    
elif runtype == "mutual":
    print ("setting initial conditions for " + filetype)
    with open('InitialConditions/data_mutual.pickle', 'r') as f:
        log_w,Gov,sigma,gamma,beta,c,l,b,omega,T,Tau_w,Tau_d,Tau_b,s,epsilon,mu,Bbar = pickle.load( f)
        #logw,Para.g,Para.ra,Para.gamma,Para.beta,c,l,b,omega,T,Tau_w,Tau_m,s,Para.epsilon,mu
    e = log_w
    e -= e.mean()
    
    N = len(e)
    C = np.mean(c)
    Gamma = np.zeros((N,6))
    mhat = c/C
    m = mhat**(sigma)
    logmhat = np.log(mhat)
    muhat = mu/m
    Psi_ = -np.mean(s*muhat) 
    
    Gamma[:,0] = logmhat
    Gamma[:,1] = logmhat
    Gamma[:,2] = muhat
    Gamma[:,3] = omega
    Gamma[:,4] = e
    Gamma[:,5] = Psi_ 
    Para.TauTarget = Tau_w
    Para.TauD = Tau_d
    if fiscaltype=='fiscal':
        Gamma = Gamma[:,:4]
elif runtype in ["htm"]:
    with open('InitialConditions/data_revision_htm.pickle', 'r') as f:
        log_w,Gov,sigma,gamma,beta,c,l,b,omega,T,Tau_w,Tau_d,Tau_b,s,htm_i,epsilon,mu = pickle.load( f)


    e = log_w
    e -= e.mean()
    N = len(e)
    
    lwedge = np.linspace(-7.,5.,100)
    Tvec = np.linspace(-4.,20.,100)
    Para.TRange = [-1,2.]
    
    lcvec = np.zeros((len(lwedge),len(Tvec)))
    for i in range(len(lwedge)):
        for j in range(len(Tvec)):
            def f(lnc):
                c = np.exp(lnc)
                uc = c**(-sigma)
                w = np.exp(lwedge[i])
                n = (uc*w)**(1./gamma)
                return Tvec[j] + n*w - c
            lcvec[i,j] = brentq(f,-40.,20.)
    Para.htm_consumption = RectBivariateSpline(lwedge,Tvec,lcvec)
    Para.TauTarget = Tau_w
    
    nothtm = htm_i < 0.5  #all agents not handtomouth
    htm = htm_i > 0.5
    N = len(e)
    uc = c**(-sigma)
    Gamma = np.zeros((N,7))
    m = 1./uc
    logm = np.log(m)
    logm[nothtm] -= logm[nothtm].mean()
    logm[htm] = (1-Tau_b)*b[htm]*beta
    muhat = mu/np.exp(logm)
    muhat[htm] = 0.
    
    Gamma[:,0] = logm
    Gamma[:,1] = logm
    Gamma[:,2] = muhat
    Gamma[:,3] = omega
    Gamma[:,4] = e
    Gamma[:,5] = s*(1-Tau_d)
    Gamma[:,6] = htm
    
    NSZ0 = [1.,1.]
    

else:
    print "Could not find type"
    exit(1)
    
    
optionaltype = ""
if len(argv) > 3:
    optionaltype ="_"
    for i in range(3,len(argv)):
        exec(argv[i])
        optionaltype = optionaltype + argv[i].replace("=", "").replace(".","")
        

if runtype == "benchmark_hetexposure":
    Para.ineq = 1.
    
## SET PARA     
y = np.exp(e)*l
delta_E = (Para.E(e)*y).mean()/y.mean()
Para.E = lambda x : np.poly1d(polyE2)(fE(x)) - delta_E
    
Para.rho_MU = rho_MU
Para.rho_Theta = rho_Theta
Para.sigma_e[:] = np.array([sigma_eta,sigma_e])
Para.sigma_E[:] = np.diag([sigma_Eps_theta_ar,sigma_epsMU])
Para.beta = beta
Para.sigma = sigma
Para.gamma = gamma
Para.epsilonbar = epsilon
Para.psi = psi
Para.Bbar = Bbar
Para.Gov = Gov
print " Using psi="
print Para.psi
Para.phat = np.array([rho_e-1.0])
Para.k = num_k
Para.ineq = ineq
Para.onlyineq = onlyineq
Para.ar = ar
Para.divMPC = divMPC
quadratic = True
Gamma0 = np.random.permutation(Gamma)
simulate.comm.Bcast([Gamma0,simulate.MPI.DOUBLE])
Para.nomalize(Gamma0)

print Para.sigma_E



###
#IRFS
##
approximate.calibrate(Para)
simulate.approximate=approximate









def computeDecomp(para,approx,approximate):
    sigma_E = approximate.sigma_E
    cov_E = sigma_E.dot(sigma_E.T)
    sigma = para.sigma
    gamma = para.gamma
    ny = len(para.y_labels)
    nY = len(para.Y_labels)
    
    iPVc = np.arange(ny)[[ylab == 'EPVc_' for ylab in para.y_labels]][0]
    iPVn = np.arange(ny)[[ylab == 'EPVn_' for ylab in para.y_labels]][0]
    ialpha = np.arange(ny)[[ylab == 'omega' for ylab in para.y_labels]][0]
    ic = np.arange(ny)[[ylab == 'c' for ylab in para.y_labels]][0]
    inl = np.arange(ny)[[ylab == 'nl' for ylab in para.y_labels]][0]
    iPVc2 = np.arange(ny)[[ylab == 'EPVc2_' for ylab in para.y_labels]][0]
    iPVn2 = np.arange(ny)[[ylab == 'EPVn2_' for ylab in para.y_labels]][0]
    iPVC = np.arange(nY)[[Ylab == 'EPVC_' for Ylab in para.Y_labels]][0]
    iPVN = np.arange(nY)[[Ylab == 'EPVN_' for Ylab in para.Y_labels]][0]
    iC = np.arange(nY)[[Ylab == 'C' for Ylab in para.Y_labels]][0]
    iN = np.arange(nY)[[Ylab == 'N' for Ylab in para.Y_labels]][0]
    
    Cbar,Nbar = approx.ss.get_Y()[[iC,iN]]
    d2Y = approx.d2Y[approximate.sigma].dot(approximate.sigma).flatten() + np.einsum('ijk,jk',approx.d2Y[sigma_E],cov_E).flatten()
    d2PVC = d2Y[iPVC]
    d2PVN = d2Y[iPVN]
    PVGammaC = 0.5*d2PVC/Cbar
    PVGammaN = 0.5*d2PVN/Nbar
    

    
    R,E,I = 0.,0.,0.
    for z,w in approx.dist:
        cbar = approx.ss.get_y(z)[ic]
        nbar = approx.ss.get_y(z)[inl]
        alpha = approx.ss.get_y(z)[ialpha]
        phi_c = alpha*(cbar**(-sigma))*cbar
        phi_n = -alpha*(nbar**gamma)*nbar
        
        d2y = approx.d2y[approximate.sigma](z).dot(approximate.sigma).flatten() + np.einsum('ijk,jk',approx.d2y[sigma_E](z),cov_E).flatten()
        d2PVc,d2PVn,d2PVc2,d2PVn2 = d2y[[iPVc,iPVn,iPVc2,iPVn2]]
        
        PVDeltaC = 0.5*(d2PVc/cbar - d2PVC/Cbar)
        PVDeltaN = 0.5*(d2PVn/nbar - d2PVN/Nbar)
        
        PVLambdaC = -0.5*(d2PVc2/cbar**2 - 2*d2PVc/cbar)
        PVLambdaN = -0.5*(d2PVn2/nbar**2 - 2*d2PVn/nbar)
        E += w*(phi_c*PVGammaC + phi_n*PVGammaN)
        R += w*(phi_c*PVDeltaC + phi_n*PVDeltaN)
        I += 0.5*w*(phi_c*sigma*PVLambdaC - phi_n*gamma*PVLambdaN )
        
    
    return R,E,I


Para.RApolicy = 1. # Compute approximations to policies with RA policy
approximate.calibrate(Para)
if runtype in ["htm"]:
    approxA = approximate.approximate(Gamma0,NSZ0)
else:
    approxA = approximate.approximate(Gamma0)
RA,EA,IA = computeDecomp(Para,approxA,approximate)

Para.RApolicy = 0.  # Compute approximations to HA optimal Policy
approximate.calibrate(Para)
if runtype in ["htm"]:
    approxB = approximate.approximate(Gamma0,NSZ0)
else:
    approxB = approximate.approximate(Gamma0)
RB,EB,IB = computeDecomp(Para,approxB,approximate)

#Total Welfare gain
DW = (RB - RA + EB - EA + IB -IA)

Efficiency = 100*(EB - EA)/DW
Redistribution = 100*(RB - RA)/DW
Insurance = 100*(IB - IA)/DW

if simulate.rank == 0:
    WelfareDecomp = np.hstack([runtype+fiscaltype,Efficiency,Redistribution,Insurance])
    
    f = open('data/main_text_table4.csv','ab')
    np.savetxt(f,WelfareDecomp,fmt='%s')
    f.close()
