# -*- coding: utf-8 -*-
import os
os.environ['OPENBLAS_NUM_THREADS'] = '1'
import numpy as np
import pandas as pd
import cPickle as pickle
import approximate
import simulate_MPI as simulate
import mkl
import getpass
from scipy import stats
mkl.set_num_threads(1)
print(simulate.rank)
import sys
import gc
"""
Default Parameters
"""
calibrationfile='calibrations/calibration_data/baseline_calibration_revision.csv'
num_k = 2600
shock = [-1.,0.]
Tshock = 1
IRFlen = 15
Nirf = 2



############################
# Load calibration         #
############################
calibration=pd.read_csv(calibrationfile).T
calibration.columns=calibration.iloc[0,:]
calibration=calibration.iloc[1:,:]


# shock process calibration
psi = calibration.psi.val
ineq = calibration.ineq_flag.val
rho_MU=calibration.rho_MU.val
rho_Theta=calibration.rho_Theta.val # only used in tfpar1 case
sigma_eps=calibration.sigma_eps.val
sigma_epsMU = calibration.sigma_epsMU.val
sigma_Eps_theta_ar = calibration.sigma_Eps_theta_ar.val
sigma_e = calibration.sigma_e.val
sigma_eta= calibration.sigma_eta.val
rho_e=calibration.rho_e.val
Gov = calibration.Gov.val
epsilon = calibration.epsilonbar.val
sigma = calibration.ra.val
gamma = calibration.gamma.val
beta = calibration.beta.val
Bbar = calibration.Bbar.val
onlyineq = 0.
ar = 1.
divMPC = 0.
YSS = None
no_idio = 0.



"""
Import script parameters
"""
argv = sys.argv
if len(argv) < 6:
    print "Need at least 5 arguments: runtype, fiscaltype, shocktype, foldername, label"
    exit(1)

runtype = argv[1]
fiscaltype = argv[2]
shocktype = argv[3]
foldername = argv[4]
label = argv[5]

print ("runtype = "+runtype)
print ("fiscaltype = "+fiscaltype)
print ("shocktype = "+shocktype)

## LOAD MODULE

if runtype in ["benchmark","benchmark_highlabor","benchmark_highdiv","benchmark_highbond","benchmark_util","benchmark_lowlabor","benchmark_lowdiv","benchmark_ce100","benchmark_aligned"]:
    if fiscaltype == "fiscal":
        import calibrations.calibration_begs2_benchmark_fiscal as Para
    elif fiscaltype == "nofiscal":
        import calibrations.calibration_begs2_benchmark_nofiscal as Para
    else:
        print "Second Argument Needs to be fiscal or nofiscal"
        exit(1)
                
elif runtype in ["benchmark_cm","benchmark_RA"]:
    if fiscaltype == "fiscal":
        import calibrations.calibration_begs2_benchmark_fiscal_cm as Para
    elif fiscaltype == "nofiscal":
        import calibrations.calibration_begs2_benchmark_nofiscal_cm as Para
    else:
        print "Second Argument Needs to be fiscal or nofiscal"
        exit(1)
       
elif runtype == "mutual":
    if fiscaltype == "fiscal":
        if shocktype == "MU":
            import calibrations.calibration_begs2_mutual_fiscal_MU as Para
        else:
            import calibrations.calibration_begs2_mutual_fiscal_TFP as Para
    elif fiscaltype == "nofiscal":
        import calibrations.calibration_begs2_mutual_nofiscal as Para
    else:
        print "Second argument for mutual needs to be fiscal or nofiscal"
    
else:
    print "First Argument Needs to be benchmark or stationary or htm or pareto"
    exit(1)

    

## SET SHOCK 

if shocktype == "MU":
    shock = [0.,-1.]
elif shocktype == "tfp":
    shock = [-1.,0.]
else:
    print "Third Argument Needs to be MU or tfp"
    exit(1)
    



recession_data=pd.read_excel('calibrations/calibration_data/recession_earnings_loss.xlsx')

recession_data=recession_data[(recession_data.percentile>0)&(recession_data.percentile<100)]
recession_data.set_index('percentile',inplace=True)

#scale correctly for near permanent shocks
ineq_f1=calibration.ineq_f1.val
ineq_f2=calibration.ineq_f2.val
ineq_f3=calibration.ineq_f3.val
ineq_scaling=calibration.ineq_scaling.val
polyE = np.polyfit(stats.norm.ppf([0.05,.5,.95])*ineq_scaling,recession_data.avg[[5,50,95]]-[ineq_f1,ineq_f2,ineq_f3],2)   #.9 scales the standard deviation of e
#Para.E = np.poly1d(polyE)

fE = lambda x: np.exp(2*x)/(1+np.exp(2*x))
polyE2 = np.polyfit(fE(stats.norm.ppf([0.05,.5,.95])*ineq_scaling),recession_data.avg[[5,50,95]]-[ineq_f1,ineq_f2,ineq_f3],2)
Para.E = lambda x : np.poly1d(polyE2)(fE(x))



## LOAD INITIAL CONDITIONS
filetype =""
if runtype in ["benchmark","benchmark_highlabor","benchmark_highdiv","benchmark_highbond","benchmark_util","benchmark_lowlabor","benchmark_lowdiv","benchmark_ce100","benchmark_aligned"]:
    print ("setting initial conditions for " + filetype)
    if runtype =="benchmark":
        with open('InitialConditions/data_benchmark.pickle', 'r') as f:
            log_w,Gov,sigma,gamma,beta,c,l,b,omega,T,Tau_w,Tau_d,Tau_b,s,epsilon,mu = pickle.load( f)
    if runtype == "benchmark_highlabor":
        with open('InitialConditions/data_benchmark_highlabor.pickle', 'r') as f:
            log_w,Gov,sigma,gamma,beta,c,l,b,omega,T,Tau_w,Tau_d,Tau_b,s,epsilon,mu = pickle.load( f)
    if runtype == "benchmark_highdiv":
        with open('InitialConditions/data_benchmark_highdiv.pickle', 'r') as f:
            log_w,Gov,sigma,gamma,beta,c,l,b,omega,T,Tau_w,Tau_d,Tau_b,s,epsilon,mu = pickle.load( f)
    if runtype == "benchmark_highbond":
        with open('InitialConditions/data_benchmark_highbond.pickle', 'r') as f:
            log_w,Gov,sigma,gamma,beta,c,l,b,omega,T,Tau_w,Tau_d,Tau_b,s,epsilon,mu = pickle.load( f)
    if runtype == "benchmark_util":
        with open('InitialConditions/data_benchmark_util.pickle', 'r') as f:
            log_w,Gov,sigma,gamma,beta,c,l,b,omega,T,Tau_w,Tau_d,Tau_b,s,epsilon,mu = pickle.load( f)
    if runtype == "benchmark_lowlabor":
        with open('InitialConditions/data_benchmark_lowlabor.pickle', 'r') as f:
            log_w,Gov,sigma,gamma,beta,c,l,b,omega,T,Tau_w,Tau_d,Tau_b,s,epsilon,mu = pickle.load( f)
    if runtype == "benchmark_lowdiv":
        with open('InitialConditions/data_benchmark_lowdiv.pickle', 'r') as f:
            log_w,Gov,sigma,gamma,beta,c,l,b,omega,T,Tau_w,Tau_d,Tau_b,s,epsilon,mu = pickle.load( f)
    if runtype == "benchmark_ce100":
        with open('InitialConditions/data_benchmark_ce100.pickle', 'r') as f:
            log_w,Gov,sigma,gamma,beta,c,l,b,omega,T,Tau_w,Tau_d,Tau_b,s,epsilon,mu = pickle.load( f)
    if runtype=="benchmark_aligned":
        with open('InitialConditions/data_benchmark_aligned.pickle', 'r') as f:
            log_w,Gov,sigma,gamma,beta,c,l,b,omega,T,Tau_w,Tau_d,Tau_b,s,epsilon,mu = pickle.load( f)

    e = log_w
    e -= e.mean()
    s_vec = (1-Tau_d)*s + Tau_d*1
    
    N = len(e)
    uc = c**(-sigma)
    Gamma = np.zeros((N,5))
    m = 1./uc
    logm = np.log(m)
    logm -= np.mean(logm)
    muhat = mu/np.exp(logm)
    
    
    Gamma[:,0] = logm
    Gamma[:,1] = muhat
    Gamma[:,2] = omega
    Gamma[:,3] = e
    Gamma[:,4] = s_vec
    Para.TauTarget = Tau_w
    
    
elif runtype=="benchmark_RA":
    #Load for parameters
    with open('InitialConditions/data_benchmark.pickle', 'r') as f:
        log_w,Gov,sigma,gamma,beta,c,l,b,omega,T,Tau_w,Tau_d,Tau_b,s,epsilon,mu = pickle.load( f)
    N = 2
    W = (epsilon-1.)/epsilon
    Tau_d = 0.
    Tau_w = 1-1./W
    Tau_b = 0.
    omega = np.ones(N)
    e = np.zeros(N)
    s_vec = np.ones(N)
    logm = np.zeros(N)
    muhat = np.zeros(N)
    
    sigma_e = 0.0
    sigma_eta= 0.0
    rho_e = 1.
    l = np.ones(N)
    
    Gamma = np.zeros((N,5))
    Gamma[:,0] = logm
    Gamma[:,1] = muhat
    Gamma[:,2] = omega
    Gamma[:,3] = e
    Gamma[:,4] = s_vec
    Para.TauTarget = Tau_w


    
elif  runtype=="benchmark_cm":
    if runtype=="benchmark_cm":
        with open('InitialConditions/data_benchmark.pickle', 'r') as f:
            log_w,Gov,sigma,gamma,beta,c,l,b,omega,T,Tau_w,Tau_d,Tau_b,s,epsilon,mu = pickle.load( f)

    e = log_w
    e -= e.mean()
    s_vec = (1-Tau_d)*s + Tau_d*1
    
    N = len(e)
    uc = c**(-sigma)
    Gamma = np.zeros((N,5))
    m = 1./uc
    logm = np.log(m)
    logm -= np.mean(logm)
    muhat = mu/np.exp(logm)
    
    Gamma[:,0] = logm
    Gamma[:,1] = muhat
    Gamma[:,2] = omega
    Gamma[:,3] = e
    Gamma[:,4] = s_vec
    
    # STY/LMP calibration
    sigma_e = 0.0
    
    sigma_eta= 0.0
    
    rho_e = 1.
    Para.TauTarget = Tau_w

    

    
elif runtype == "mutual" or runtype == "mutual_decomp":
    print ("setting initial conditions for " + filetype)
    with open('InitialConditions/data_mutual.pickle', 'r') as f:
        log_w,Gov,sigma,gamma,beta,c,l,b,omega,T,Tau_w,Tau_d,Tau_b,s,epsilon,mu,Bbar = pickle.load( f)
        #logw,Para.g,Para.ra,Para.gamma,Para.beta,c,l,b,omega,T,Tau_w,Tau_m,s,Para.epsilon,mu
    e = log_w
    e -= e.mean()
    
    N = len(e)
    C = np.mean(c)
    Gamma = np.zeros((N,5))
    mhat = c/C
    m = mhat**(sigma)
    logmhat = np.log(mhat)
    muhat = mu/m
    Psi_ = -np.mean(s*muhat) 
    
    Gamma[:,0] = logmhat
    Gamma[:,1] = muhat
    Gamma[:,2] = omega
    Gamma[:,3] = e
    Gamma[:,4] = Psi_ 
    Para.TauTarget = Tau_w
    Para.TauD = Tau_d
    if fiscaltype=='fiscal':
        Gamma = Gamma[:,:4]

else:
    print "Could not find type"
    exit(1)
    
    
optionaltype = ""
if len(argv) > 6:
    optionaltype ="_"
    for i in range(6,len(argv)):
        exec(argv[i])
        optionaltype = optionaltype + argv[i].replace("=", "").replace(".","")
        

    
if no_idio:
    sigma_e = 0.0

    sigma_eta= 0.0

    rho_e = 1.

    
    
filetype = runtype+"_"+shocktype+"_"+fiscaltype+optionaltype

print "saving simulations to filename:"
print filetype
print Para

    
## SET PARA     
y = np.exp(e)*l
delta_E = (Para.E(e)*y).mean()/y.mean()
Para.E = lambda x : np.poly1d(polyE2)(fE(x)) - delta_E
    
Para.rho_MU = rho_MU
Para.rho_Theta = rho_Theta
Para.sigma_e[:] = np.array([sigma_eta,sigma_e])
Para.sigma_E[:] = np.diag([sigma_Eps_theta_ar,sigma_epsMU])
Para.beta = beta
Para.sigma = sigma
Para.gamma = gamma
Para.epsilonbar = epsilon
Para.psi = psi
Para.Bbar = Bbar
Para.Gov = Gov
print " Using psi="
print Para.psi
Para.phat = np.array([rho_e-1.0])
Para.k = num_k
Para.ineq = ineq
Para.onlyineq = onlyineq
Para.ar = ar
Para.divMPC = divMPC
quadratic = True
Gamma0 = np.random.permutation(Gamma)
simulate.comm.Bcast([Gamma0,simulate.MPI.DOUBLE])
Para.nomalize(Gamma0)

print Para.sigma_E



###
#IRFS
##
approximate.calibrate(Para)
simulate.approximate=approximate
T = IRFlen+Tshock
if YSS is not None:
    approximate.steadystate.Y0 = YSS

def generate_irf(ix):
    Gamma,Z,Y,Shocks,y = {},{},{},{},{}
    Gamma[0] = Gamma0
    
    agg_shocks = np.random.randn(T,Para.nEps)
    
    approx = approximate.approximate(Gamma0,fit=False)
    
    Z[0] = approx.ss.get_Y()[:Para.nZ]#np.array([0.])
    
    
    
    simulate.simulate_aggstate(Para,Gamma,Z,Y,Shocks,y,Tshock,quadratic=quadratic,agg_shocks=agg_shocks)
    
    rand_state = np.random.get_state()
    agg_shocks[Tshock,:] = 0.
    simulate.simulate_aggstate(Para,Gamma,Z,Y,Shocks,y,T,T0=Tshock-1,agg_shocks=agg_shocks,quadratic=quadratic)
    if approximate.rank == 0:
        Y2 = pd.DataFrame(np.vstack(Y.values()),columns=Para.Y_labels)
        if runtype == "mutual" or runtype == "mutualdata" or  runtype ==  "mutual_s" or runtype == "htm":
            Y2['i'] = 1/Y2['Q']
        else:
            Y2['Alpha1'] = Y2.Alpha1_.shift(-1)
            Y2['i'] = Y2.Alpha2/(Para.beta*Y2.Alpha1)
        Y2['R'] = Y2.i.shift()/(1+Y2.Pi)
        Y2['R'][0] = 0.
        
    agg_shocks[Tshock,:] = shock
    
    #uses same random state
    np.random.set_state(rand_state)
    
    simulate.simulate_aggstate(Para,Gamma,Z,Y,Shocks,y,T,T0=Tshock-1,agg_shocks=agg_shocks,quadratic=quadratic)
    
    if approximate.rank == 0:
        Y1 = pd.DataFrame(np.vstack(Y.values()),columns=Para.Y_labels)
        if runtype == "mutual" or runtype == "mutualdata" or  runtype ==  "mutual_s" or runtype == "htm":
            Y1['i'] = 1/Y1.Q
        else:
            Y1['Alpha1'] = Y1.Alpha1_.shift(-1)
            Y1['i'] = Y1.Alpha2/(Para.beta*Y1.Alpha1)
        Y1['R'] = Y1.i.shift()/(1+Y1.Pi)
        Y1['R'][0] = 0.
    
    if approximate.rank == 0:
        print Y1['Tau']-Y2['Tau']
        print Y1['Pi']-Y2['Pi']
        with open('data/rawdata/'+foldername+'/'+filetype+'/irf_'+getpass.getuser()+"_"+str(ix)+'.pickle', 'w') as f:
            pickle.dump((Y1,Y2),f)
            f.close()
         
                                                #variables to scale by 100 for %
def saveIRF(var=['Tau','Pi','i','W', 'D','lY']):
    fullpath = os.path.join('./data/','rawdata',foldername,filetype)
    filenames=[f for f in os.listdir(fullpath) if f.endswith('.pickle')]
    
    Yimpulse_data=[]
    N=len(filenames)
    for n in range(N):
        print (filenames[n])
        with open(os.path.join(fullpath,filenames[n]), 'r') as f:
            Y1,Y2=pd.read_pickle(f,compression=None)
            
            
        Y1['lY']=np.log(Y1.Y)
        Y2['lY']=np.log(Y2.Y)
        if runtype != 'mutual':
            Y1.Tr = Y1.Tr + Tau_d*Y1.D #adjust for dividend tax
            Y2.Tr = Y2.Tr + Tau_d*Y2.D
        Y1.Tr = Y1.Tr/Y1.Y[0]#Compute transfers relative to initial output
        Y2.Tr = Y2.Tr/Y1.Y[0]

        Yimpulse_data.append(Y1-Y2)
    
    Yimpulse = sum(Yimpulse_data)/len(Yimpulse_data)
    Yimpulse['date']=range(len(Yimpulse))
    
    Yimpulse[var]=Yimpulse[var]*100
    
    Yimpulse = Yimpulse.add_suffix('_'+label).rename(columns={'date_'+label:'date'})
    Yimpulse.to_csv('data/rawdata/'+foldername+'/'+filetype+'.csv')
            
if approximate.rank == 0:
    try:
        os.mkdir('data/rawdata/')
    except OSError:
        pass
    try:
        os.mkdir('data/rawdata/'+foldername+"/")
    except OSError:
        pass
    try:
        os.mkdir('data/rawdata/'+foldername+"/"+filetype+"/")
    except OSError:
        pass

## RUN IRFS            
for i in range(Nirf):
    generate_irf(i)
    gc.collect()
    
    
if approximate.rank == 0:
    saveIRF()
    




