# -*- coding: utf-8 -*-
"""
Created on Fri Dec 16 13:01:27 2016

@author: david,anmol

Calibrating wages
"""
import os
os.environ['OPENBLAS_NUM_THREADS'] = '1'
import numpy as np
import pandas as pd
from scipy.interpolate import RectBivariateSpline
from scipy.optimize import root
import cPickle as pickle
import approximate_NSZ as approximate
import simulate_MPI_NSZ as simulate
import mkl
import getpass
from scipy import stats
from scipy.optimize import brentq
mkl.set_num_threads(1)
print(simulate.rank)
import sys
import gc
"""
Default Parameters
"""
calibrationfile='calibrations/calibration_data/baseline_calibration_revision.csv'
num_k = 4000
shock = [-1.,0.]
Tshock = 1
IRFlen = 15
Nirf = 5



############################
# Load calibration         #
############################
calibration=pd.read_csv(calibrationfile).T
calibration.columns=calibration.iloc[0,:]
calibration=calibration.iloc[1:,:]


# shock process calibration
psi = calibration.psi.val
ineq = calibration.ineq_flag.val
rho_MU=calibration.rho_MU.val
rho_Theta=calibration.rho_Theta.val # only used in tfpar1 case
sigma_eps=calibration.sigma_eps.val
sigma_epsMU = calibration.sigma_epsMU.val
sigma_Eps_theta_ar = calibration.sigma_Eps_theta_ar.val#0.04
sigma_e = calibration.sigma_e.val#.22#0.25#10 #.17
sigma_eta= calibration.sigma_eta.val#0.25#0.095 #0.17
rho_e=calibration.rho_e.val
Gov = calibration.Gov.val
epsilon = calibration.epsilonbar.val
sigma = calibration.ra.val
gamma = calibration.gamma.val
beta = calibration.beta.val
Bbar = calibration.Bbar.val
onlyineq = 0.
ar = 1.
divMPC = 0.
no_idio = 0.
YSS = None

"""
Import script parameters
"""
argv = sys.argv
if len(argv) < 6:
    print "Need at least 5 arguments: runtype, fiscaltype, shocktype, foldername, label"
    exit(1)

runtype = argv[1]
fiscaltype = argv[2]
shocktype = argv[3]
foldername = argv[4]
label = argv[5]

print ("runtype = "+runtype)
print ("fiscaltype = "+fiscaltype)
print ("shocktype = "+shocktype)

## LOAD MODULE

        
if runtype ==  "revision_htm_hetMPC":
    NSZ0 = np.array([1.,0.])
    import calibrations.calibration_begs2_revision_htm_nofiscal_het_NSZ as Para
             
elif runtype == "revision_htm" or runtype == "revision_htm_poorcih":
    NSZ0 = np.array([1.])
    import calibrations.calibration_begs2_revision_htm_nofiscal_NSZ as Para
else:
    print "First Argument Needs to be revision_htm_hetMPC or revision_htm"
    exit(1)

    

## SET SHOCK 

if shocktype == "MU":
    shock = [0.,-1.]
elif shocktype == "tfp":
    shock = [-1.,0.]
else:
    print "Third Argument Needs to be MU or tfp or tfpar1"
    exit(1)
    



recession_data=pd.read_excel('calibrations/calibration_data/recession_earnings_loss.xlsx')

recession_data=recession_data[(recession_data.percentile>0)&(recession_data.percentile<100)]
recession_data.set_index('percentile',inplace=True)

#scale correctly for near permanent shocks
ineq_f1=calibration.ineq_f1.val
ineq_f2=calibration.ineq_f2.val
ineq_f3=calibration.ineq_f3.val
ineq_scaling=calibration.ineq_scaling.val
polyE = np.polyfit(stats.norm.ppf([0.05,.5,.95])*ineq_scaling,recession_data.avg[[5,50,95]]-[ineq_f1,ineq_f2,ineq_f3],2)   #.9 scales the standard deviation of e
#Para.E = np.poly1d(polyE)

fE = lambda x: np.exp(2*x)/(1+np.exp(2*x))
polyE2 = np.polyfit(fE(stats.norm.ppf([0.05,.5,.95])*ineq_scaling),recession_data.avg[[5,50,95]]-[ineq_f1,ineq_f2,ineq_f3],2)
Para.E = lambda x : np.poly1d(polyE2)(fE(x))



## LOAD INITIAL CONDITIONS
filetype =""
if runtype in ["revision_htm","revision_htm_poorcih","revision_htm_hetMPC"]:
    if runtype == "revision_htm" or runtype== "revision_htm_hetMPC":
        with open('InitialConditions/data_revision_htm.pickle', 'r') as f:
                log_w,Gov,sigma,gamma,beta,c,l,b,omega,T,Tau_w,Tau_d,Tau_b,s,htm_i,epsilon,mu = pickle.load( f)
    else:
        with open('InitialConditions/data_revision_htm_poorcih.pickle', 'r') as f:
                log_w,Gov,sigma,gamma,beta,c,l,b,omega,T,Tau_w,Tau_d,Tau_b,s,htm_i,epsilon,mu = pickle.load( f)

    e = log_w
    e -= e.mean()
    N = len(e)
    
    lwedge = np.linspace(-7.,5.,100)
    Tvec = np.linspace(-4.,20.,100)
    Para.TRange = [-1,2.]
    
    lcvec = np.zeros((len(lwedge),len(Tvec)))
    for i in range(len(lwedge)):
        for j in range(len(Tvec)):
            def f(lnc):
                c = np.exp(lnc)
                uc = c**(-sigma)
                w = np.exp(lwedge[i])
                n = (uc*w)**(1./gamma)
                return Tvec[j] + n*w - c
            lcvec[i,j] = brentq(f,-40.,20.)
    Para.htm_consumption = RectBivariateSpline(lwedge,Tvec,lcvec)
    Para.TauTarget = Tau_w
    
    nothtm = htm_i < 0.5  #all agents not handtomouth
    htm = htm_i > 0.5
    N = len(e)
    uc = c**(-sigma)
    Gamma = np.zeros((N,6))
    m = 1./uc
    logm = np.log(m)
    logm[nothtm] -= logm[nothtm].mean()
    logm[htm] = (1-Tau_b)*b[htm]*beta
    muhat = mu/np.exp(logm)
    muhat[htm] = 0.
    
    Gamma[:,0] = logm
    Gamma[:,1] = muhat
    Gamma[:,2] = omega
    Gamma[:,3] = e
    Gamma[:,4] = s*(1-Tau_d)
    Gamma[:,5] = htm
    
    Gamma = np.vstack((Gamma,Gamma))
    

else:
    print "Could not find type"
    exit(1)
    
    
optionaltype = ""
if len(argv) > 6:
    optionaltype ="_"
    for i in range(6,len(argv)):
        exec(argv[i])
        optionaltype = optionaltype + argv[i].replace("=", "").replace(".","")
        

    
filetype = runtype+"_"+shocktype+"_"+fiscaltype+optionaltype

print "saving simulations to filename:"
print filetype
print Para

if no_idio == 1:
    sigma_e = 0.0

    sigma_eta= 0.0
    
    rho_e = 1.

    
## SET PARA     
y = np.exp(e)*l
delta_E = (Para.E(e)*y).mean()/y.mean()
Para.E = lambda x : np.poly1d(polyE2)(fE(x)) - delta_E
    
Para.rho_MU = rho_MU
Para.rho_Theta = rho_Theta
Para.sigma_e[:] = np.array([sigma_eta,sigma_e])
Para.sigma_E[:] = np.diag([sigma_Eps_theta_ar,sigma_epsMU])
Para.beta = beta
Para.sigma = sigma
Para.gamma = gamma
Para.epsilonbar = epsilon
Para.psi = psi
Para.Bbar = Bbar
Para.Gov = Gov
print " Using psi="
print Para.psi
Para.phat = np.array([rho_e-1.0])
Para.k = num_k
Para.ineq = ineq
Para.onlyineq = onlyineq
Para.ar = ar
Para.divMPC = divMPC
quadratic = True
Gamma0 = np.random.permutation(Gamma)
simulate.comm.Bcast([Gamma0,simulate.MPI.DOUBLE])
Para.nomalize(Gamma0)

print Para.sigma_E

###
#IRFS
##
approximate.calibrate(Para)
simulate.approximate=approximate
T = IRFlen+Tshock
if YSS is not None:
    approximate.steadystate.Y0 = YSS

def generate_irf(ix):
    Gamma,Z,Y,Shocks,y = {},{},{},{},{}
    Gamma[0] = Gamma0
    
    agg_shocks = np.random.randn(T,Para.nEps)
    
    approx = approximate.approximate(Gamma0,NSZ0,fit=False)
    
    Z[0] = approx.ss.get_Y()[:Para.nZ]#np.array([0.])
    
    
    
    simulate.simulate_aggstate(Para,Gamma,Z,Y,Shocks,y,Tshock,quadratic=quadratic,agg_shocks=agg_shocks)
    
    rand_state = np.random.get_state()
    agg_shocks[Tshock,:] = 0.
    simulate.simulate_aggstate(Para,Gamma,Z,Y,Shocks,y,T,T0=Tshock-1,agg_shocks=agg_shocks,quadratic=quadratic)
    if approximate.rank == 0:
        Y2 = pd.DataFrame(np.vstack(Y.values()),columns=Para.Y_labels)
        if runtype == "mutual" or runtype == "mutualdata" or  runtype ==  "mutual_s" or runtype == "htm":
            Y2['i'] = 1/Y2['Q']
        else:
            Y2['Alpha1'] = Y2.Alpha1_.shift(-1)
            Y2['i'] = Y2.Alpha2/(Para.beta*Y2.Alpha1)
        Y2['R'] = Y2.i.shift()/(1+Y2.Pi)
        Y2['R'] = 0.
        
    agg_shocks[Tshock,:] = shock
    
    #uses same random state
    np.random.set_state(rand_state)
    
    simulate.simulate_aggstate(Para,Gamma,Z,Y,Shocks,y,T,T0=Tshock-1,agg_shocks=agg_shocks,quadratic=quadratic)
    
    if approximate.rank == 0:
        Y1 = pd.DataFrame(np.vstack(Y.values()),columns=Para.Y_labels)
        if runtype == "mutual" or runtype == "mutualdata" or  runtype ==  "mutual_s" or runtype == "htm":
            Y1['i'] = 1/Y1.Q
        else:
            Y1['Alpha1'] = Y1.Alpha1_.shift(-1)
            Y1['i'] = Y1.Alpha2/(Para.beta*Y1.Alpha1)
        Y1['R'] = Y1.i.shift()/(1+Y1.Pi)
        Y1['R'][0] = 0.

    
    if approximate.rank == 0:
        print Y1['Tau']-Y2['Tau']
        print Y1['Pi']-Y2['Pi']
        with open('data/rawdata/'+foldername+'/'+filetype+'/irf_'+getpass.getuser()+"_"+str(ix)+'.pickle', 'w') as f:
            pickle.dump((Y1,Y2),f)
            f.close()








               #variables to scale by 100 for %
def saveIRF(var=['Tau','Pi','i','W', 'D','lY']):
    fullpath = os.path.join('./data/','rawdata',foldername,filetype)
    filenames=[f for f in os.listdir(fullpath) if f.endswith('.pickle')]
    
    Yimpulse_data=[]
    N=len(filenames)
    for n in range(N):
        print (filenames[n])
        with open(os.path.join(fullpath,filenames[n]), 'r') as f:
            Y1,Y2=pd.read_pickle(f,compression=None)
            
            
        Y1['lY']=np.log(Y1.Y)
        Y2['lY']=np.log(Y2.Y)
        if runtype != 'mutual':
            Y1.Tr = Y1.Tr + Tau_d*Y1.D #adjust for dividend tax
            Y2.Tr = Y2.Tr + Tau_d*Y2.D
        Y1.Tr = Y1.Tr/Y1.Y[0]#Compute transfers relative to initial output
        Y2.Tr = Y2.Tr/Y1.Y[0]

        Yimpulse_data.append(Y1-Y2)
    
    Yimpulse = sum(Yimpulse_data)/len(Yimpulse_data)
    Yimpulse['date']=range(len(Yimpulse))
    
    Yimpulse[var]=Yimpulse[var]*100
    
    Yimpulse = Yimpulse.add_suffix('_'+label).rename(columns={'date_'+label:'date'})
    Yimpulse.to_csv('data/rawdata/'+foldername+'/'+filetype+'.csv')
            
if approximate.rank == 0:
    try:
        os.mkdir('data/rawdata/')
    except OSError:
        pass
    try:
        os.mkdir('data/rawdata/'+foldername+"/")
    except OSError:
        pass
    try:
        os.mkdir('data/rawdata/'+foldername+"/"+filetype+"/")
    except OSError:
        pass

## RUN IRFS            
for i in range(Nirf):
    generate_irf(i)
    gc.collect()
    
    
if approximate.rank == 0:
    saveIRF()
    

