# -*- coding: utf-8 -*-
import numpy as np
import stats
import statsmodels.api as sm 
import pandas as pd
import static_case_shares
import static_case_htm
from scipy import optimize
import cPickle as pickle
import time, sys
from IPython.display import clear_output



argv = sys.argv
if len(argv) < 2:
    print "Need at least 1 arguments"
    exit(1)

htmtype = argv[1]



## INPUTS
np.random.seed(32434)
N = 100000
min_hours=100
min_wages=0.01

chi_factor=2.37 ## explain what this is

if htmtype == 'htm':
    slope_htm=-0.35
    intercept_htm=0.75
    pistaferri=[0.64,0.6,0.57,0.53,0.5,0.48,0.45,0.40,.33]
    savefilename='calibrations/calibration_data/init_data_scf_htm.csv'    
elif htmtype == 'nohtm':
    slope_htm=0.0
    intercept_htm=0.0
    pistaferri= np.zeros(9)
    savefilename='calibrations/calibration_data/init_data_scf_nohtm.csv'    
else:
    print('no htm type found')
    
scfdatafilename='calibrations/calibration_data/scf.csv'



scf = pd.read_csv(scfdatafilename)
scf['BOND']=scf.GBMUTF+scf.NOTXBND+scf.GOVTBND+scf.LIQ+scf.SAVBND+scf.IRAKHBND+scf.TFBMUTF+scf.OBMUTF-scf.CCBAL-scf.ODEBT
scf['STOCK']=scf.NMMF+scf.STOCKS+scf.RETQLIQ+scf.OTHMA+scf.OTHFIN
scf['BOND']=scf['BOND']/1000 # bonds ar measured in 1000 dollarssc
scf['STOCK']=scf['STOCK']/1000 # bonds ar measured in 1000 dollarssc
scf['FinWealth']=scf['BOND']+scf['STOCK']
wages=np.zeros(len(scf))
wages[scf.index[scf.HOURS>min_hours].values]=((scf[scf.HOURS>min_hours]['WAGEINC']+.00000000001)/scf[scf.HOURS>min_hours]['HOURS'])
scf['WAGES']=wages
scf['WAGES']=scf['WAGES']*stats.mean(scf.HOURS,scf.WGT)/1000
scf['lnWages']=np.log(scf['WAGES'])
selection=(scf.SPOUSE==2)&(scf.HOURS>min_hours)&(scf.WAGES>min_wages)
maxstock=stats.quantile_1D(scf[selection].STOCK,scf[selection].WGT,0.995)
maxbond=stats.quantile_1D(scf[selection].STOCK,scf[selection].WGT,0.995)
selection=(scf.SPOUSE==2)&(scf.HOURS>min_hours)&(scf.WAGES>min_wages)&(scf.STOCK<maxstock)
scf['n_wgts']=scf['WGT']/scf[selection].WGT.sum()
scf['SHARES']=scf['STOCK']/(scf[selection].n_wgts*scf[selection].STOCK).sum()
scf['NORM_WAGES'] = scf.WAGES/(scf[selection].WAGES*scf[selection].n_wgts).sum()
scf['NORM_BOND'] = scf.BOND/(scf[selection].WAGES*scf[selection].n_wgts).sum()
scf['NORM_FinWealth']=scf.FinWealth/(scf[selection].WAGES*scf[selection].n_wgts).sum()
scf['SHARES_FINW']=scf['FinWealth']/(scf[selection].n_wgts*scf[selection].FinWealth).sum()




print "----------------------"
print "mean household wage (1000 $ for avg annual hrs)"
print stats.mean(scf[selection]['WAGES'],scf[selection]['WGT'])
print "----------------------"
print "mean bond holding (in 1000$)"
print  stats.mean(scf[selection]['BOND'],scf[selection]['WGT'])
print "----------------------"
print "mean stock holding (in 1000$)"
print  stats.mean(scf[selection]['STOCK'],scf[selection]['WGT'])
print "----------------------"
print "std of household ln wage "
print (stats.variance(scf[selection]['lnWages'],scf[selection]['WGT'])**0.5)
print "----------------------"
print "std of household bonds/wage "
print (stats.variance(scf[selection]['NORM_BOND'],scf[selection]['WGT'])**0.5)
print "----------------------"
print "std of household SHARES"
print (stats.variance(scf[selection]['SHARES'],scf[selection]['WGT'])**0.5)



print "----------------------"
print "corr household ln wage, bonds "
cov=stats.covariance(scf[selection]['lnWages'],scf[selection]['BOND'],scf[selection]['WGT'])
var_lnwages=stats.variance(scf[selection]['lnWages'],scf[selection]['WGT'])
var_bonds=stats.variance(scf[selection]['BOND'],scf[selection]['WGT'])
print ( cov/(var_lnwages*var_bonds)**0.5 )  


print "----------------------"
print "corr household stocks, bonds "
cov=stats.covariance(scf[selection]['STOCK'],scf[selection]['BOND'],scf[selection]['WGT'])
var_STOCK=stats.variance(scf[selection]['STOCK'],scf[selection]['WGT'])
var_bonds=stats.variance(scf[selection]['BOND'],scf[selection]['WGT'])
print ( cov/(var_STOCK*var_bonds)**0.5 )  



print "----------------------"
print "corr household stocks, ln wages"
cov=stats.covariance(scf[selection]['lnWages'],scf[selection]['STOCK'],scf[selection]['WGT'])
var_STOCK=stats.variance(scf[selection]['STOCK'],scf[selection]['WGT'])
var_lnWages=stats.variance(scf[selection]['lnWages'],scf[selection]['WGT'])
print ( cov/(var_STOCK*var_lnWages)**0.5 )  



pop=scf[selection].WGT.sum()
print "----------------------"
print "selection sample/ pop" 
print pop/scf.WGT.sum()
print "----------------------"

non_stockholders=scf[(scf.SPOUSE==2)&(scf.HOURS>min_hours)&(scf.WAGES>min_wages)&(scf.STOCK<0.001)].WGT.sum()

print "----------------------"
print "non_stockholders/selected sample" 
print non_stockholders/pop
print "----------------------"

stock_holders=(scf.SPOUSE==2)&(scf.HOURS>min_hours)&(scf.WAGES>min_wages)&(scf.STOCK>0.001)&(scf.STOCK<maxstock)

Q_finer=np.linspace(0.0,1.0,101)
qstocks_finer=[stats.quantile_1D(scf[stock_holders].STOCK,scf[stock_holders].WGT,q) for q in Q_finer]
stockholder_distr=pd.DataFrame(Q_finer.T,columns=['percentiles'])
stockholder_distr['val']=qstocks_finer


#hist,bin_edges=np.histogram(scf[stock_holders]['STOCK'],weights=scf[stock_holders]['WGT'],bins=stock_quantiles[stock_quantiles.val>0.001].val )
#histnormalized=hist/hist.sum()


# regressions


X=(scf[selection][['NORM_WAGES']])
YB=(scf[selection]['NORM_BOND'])
X=sm.add_constant(X)
wls_model = sm.WLS(YB,X, weights=scf[selection].WGT)
reg1 = wls_model.fit()
print reg1.summary()

intercept_reg1=reg1.params.const
slope_w_reg1=reg1.params.NORM_WAGES
std_resid_reg1=stats.variance(reg1.resid,scf[selection].WGT)**0.5



X=(scf[selection][['lnWages']])
YB=(scf[selection]['NORM_BOND'])
X=sm.add_constant(X)
wls_model = sm.WLS(YB,X, weights=scf[selection].WGT)
reg1b = wls_model.fit()
print reg1b.summary()

intercept_reg1b=reg1b.params.const
slope_w_reg1b=reg1b.params.lnWages
std_resid_reg1b=stats.variance(reg1b.resid,scf[selection].WGT)**0.5




X=(scf[selection][['NORM_WAGES','SHARES']])
YB=(scf[selection]['NORM_BOND'])
X=sm.add_constant(X)
wls_model = sm.WLS(YB,X, weights=scf[selection].WGT)
reg2= wls_model.fit()
print reg2.summary()
std_resid_reg2=stats.variance(reg2.resid,scf[selection].WGT)**0.5

intercept_reg2=reg2.params.const
slope_w_reg2=reg2.params.NORM_WAGES
slope_s_reg2=reg2.params.SHARES



X=(scf[selection][['lnWages','SHARES']])
YB=(scf[selection]['NORM_BOND'])
X=sm.add_constant(X)
wls_model = sm.WLS(YB,X, weights=scf[selection].WGT)
reg2b= wls_model.fit()
print reg2b.summary()
intercept_reg2b=reg2b.params.const
slope_w_reg2b=reg2b.params.lnWages
slope_s_reg2b=reg2b.params.SHARES
std_resid_reg2b=stats.variance(reg2b.resid,scf[selection].WGT)**0.5




X=(scf[selection][['SHARES']])
YB=(scf[selection]['NORM_WAGES'])
X=sm.add_constant(X)
wls_model = sm.WLS(YB,X, weights=scf[selection].WGT)
reg3 = wls_model.fit()
print reg3.summary()

intercept_reg3=reg3.params.const
slope_w_s_reg3=reg3.params.SHARES
std_resid_reg3=stats.variance(reg3.resid,scf[selection].WGT)**0.5



X=(scf[selection][['SHARES']])
YB=(scf[selection]['lnWages'])
X=sm.add_constant(X)
wls_model = sm.WLS(YB,X, weights=scf[selection].WGT)
reg3b = wls_model.fit()
print reg3b.summary()

intercept_reg3b=reg3b.params.const
slope_w_s_reg3b=reg3b.params.SHARES
std_resid_reg3b=stats.variance(reg3b.resid,scf[selection].WGT)**0.5


##### INPUTS FOR DAVID ########
# stocks
# fraction of people who dont hold stocks
fraction_nonstockholders= non_stockholders/pop
# percentiles of the stockholder distribution. I dropped the last entry and create 100 bins 0-99
#stockholder_distr=stockholder_distr[:-1]


# bonds
# reg1
intercept_reg1=reg1.params.const
slope_w_reg1=reg1.params.NORM_WAGES
# reg2
intercept_reg2=reg2.params.const
slope_w_reg2=reg2.params.NORM_WAGES
slope_s_reg2=reg2.params.SHARES

scf_selection = scf[selection]


def draw_population(N,equal):
    s = np.ones(N)
    htm = np.zeros(N)
    cih = np.zeros(N)
    ns = np.random.choice(scf[selection].index,N,p=scf[selection].n_wgts)
    
    b = np.array(scf.NORM_BOND[ns])
    logw = np.array(scf.lnWages[ns])
    logw = logw - logw.mean()
    s= np.array(scf.SHARES[ns])
    s_mf = np.array(scf.SHARES_FINW[ns])
        
    
    s = s/s.mean()
    s_mf=s_mf/s_mf.mean()
    
    #b=b*(1-htm)
    cih=b+chi_factor*s+np.exp(logw)
    cihq = pd.qcut(cih,9,labels=False)
    for i in range(N):
        if np.random.rand() < pistaferri[cihq[i]]:
            htm[i]=1
    
    return logw,s,b,s_mf,htm,cih



logw,s,b,s_mf,htm,cih = draw_population(N,False) 
ydata=pd.DataFrame([logw,s,b,s_mf,htm,cih] ).T
ydata.columns=['log_we','s_i','b','s_mf','htm','cih']
ydata['w_e']=np.exp(ydata.log_we)    
ydata.to_csv(savefilename)

print ydata[['htm','cih']].groupby(pd.cut(ydata['cih'],map(lambda t: ydata['cih'].quantile(t), np.linspace(.01,1.0,10)  ))).mean()
        

        
    
print pistaferri
