# -*- coding: utf-8 -*-
import sys
import calibrations.calibration_begs2_CE_ar1_taxes as Para
import numpy as np
#np.set_printoptions(threshold=np.nan)
import pandas as pd
import cPickle as pickle
import approximate
import simulate_MPI as simulate
import mkl
import os
from scipy import stats
mkl.set_num_threads(1)
print(simulate.rank)

def setseed(seed):
    np.random.seed(seed)
    for i in range(approximate.rank+1):
        seed = np.random.randint(1000000)



############################
# Load Iputs               #
############################
#Argument 1: Length of simulation
argv = sys.argv
if len(argv) < 3:
    print "Need at least 2 arguments: length and name"
    exit(1)
Tnum= int(argv[1])
runtype = argv[2]

calibrationfile='calibrations/calibration_data/baseline_calibration_revision.csv'
InitialConditionsfilename='InitialConditions/data_benchmark.pickle'
############################
# Load calibration         #
############################
calibration=pd.read_csv(calibrationfile).T
calibration.columns=calibration.iloc[0,:]
calibration=calibration.iloc[1:,:]


# shock process calibration
psi = calibration.psi.val
ineq = calibration.ineq_flag.val
rho_MU=calibration.rho_MU.val
rho_Theta=calibration.rho_Theta.val # only used in tfpar1 case
sigma_eps=calibration.sigma_eps.val
sigma_epsMU = calibration.sigma_epsMU.val
sigma_Eps_theta_ar = calibration.sigma_Eps_theta_ar.val
sigma_e = calibration.sigma_e.val
sigma_eta= calibration.sigma_eta.val
rho_e=calibration.rho_e.val
Bbar = 1.98#see mutual fund calibration
Gov = calibration.Gov.val
onlyineq = 0.
ar = 1.
YSS = None
num_k=1200
b_taylor = 2.5
if runtype == "appendix_figure_26":
    ineq = 1.



recession_data=pd.read_excel('calibrations/calibration_data/recession_earnings_loss.xlsx')

recession_data=recession_data[(recession_data.percentile>0)&(recession_data.percentile<100)]
recession_data.set_index('percentile',inplace=True)

#scale correctly for near permanent shocks
ineq_f1=calibration.ineq_f1.val
ineq_f2=calibration.ineq_f2.val
ineq_f3=calibration.ineq_f3.val
ineq_scaling=calibration.ineq_scaling.val

fE = lambda x: np.exp(2*x)/(1+np.exp(2*x))
polyE = np.polyfit(fE(stats.norm.ppf([0.05,.5,.95])*ineq_scaling),recession_data.avg[[5,50,95]]-[ineq_f1,ineq_f2,ineq_f3],2)



############################
# Load initial conditions  #
############################
with open(InitialConditionsfilename, 'r') as f:
    log_w,Gov,sigma,gamma,beta,c,l,b,omega,T,Tau_w,Tau_d,Tau_b,s,epsilon,mu = pickle.load( f)


e = log_w
e -= e.mean()
s_vec = s
uc = c**(-sigma)
m = 1./uc
logm = np.log(m)
logm -= np.mean(logm)

##############
# Set up model  #
##############
N = len(e)
Gamma = np.zeros((N,4))
Gamma[:,0] = logm
Gamma[:,1] = e
Gamma[:,2] = s_vec
Gamma[:,3] = 0.

Gamma = np.vstack([Gamma,Gamma])


Para.E = lambda x : np.poly1d(polyE)(fE(x))
Para.rho_MU = calibration.rho_MU.val
Para.rho_Theta = calibration.rho_Theta.val
Para.sigma_E = np.diag([0.,sigma_epsMU,sigma_Eps_theta_ar])
Para.sigma_e[:] = np.array([sigma_eta,sigma_e])
Para.beta = beta
Para.sigma = sigma
Para.gamma = gamma
Para.epsilonbar = epsilon
Para.psi = psi
Para.Bbar = Bbar
Para.Gov = Gov
print " Using psi="
print Para.psi
Para.phat = np.array([rho_e-1.0])
######

Para.k = num_k
Para.ineq = ineq
Para.onlyineq = onlyineq
Para.ar = ar
Para.TauTarget = calibration.TauTarget.val
Para.TaubTarget = calibration.TaubTarget.val
Para.TaudTarget = calibration.TaudTarget.val
Para.b_taylor = b_taylor
quadratic = True
Gamma0 = np.random.permutation(Gamma)
simulate.comm.Bcast([Gamma0,simulate.MPI.DOUBLE])
Para.nomalize(Gamma0)

if ineq == 1.:
    y = np.exp(log_w)*l
    delta_E = (Para.E(e)*y).mean()/y.mean()
    Para.E = lambda x : np.poly1d(polyE)(fE(x)) - delta_E

print Para.E(-1.),Para.E(0.),Para.E(1.)
print Para.sigma_E
print Para.sigma_e
print ineq

############################
# Simulate                 #
############################
setseed(342423423)
approximate.calibrate(Para)
simulate.approximate=approximate
T = Tnum
if YSS is not None:
    approximate.steadystate.Y0 = YSS

    
Gamma,Z,Y,Shocks,y = {},{},{},{},{}
Gamma[0] = Gamma0
approximate.calibrate(Para)
agg_shocks = np.random.normal(0,1,(Tnum,Para.nEps))
approx = approximate.approximate(Gamma0,fit=False)
Z[0] = approx.ss.get_Y()[:Para.nZ]
simulate.approximate=approximate
simulate.simulate_aggstate(Para,Gamma,Z,Y,Shocks,y,T,quadratic=quadratic,agg_shocks=agg_shocks)


############################
# Save results             #
############################
if approximate.rank == 0:
    try:
        os.mkdir('data/rawdata/'+runtype)
    except OSError:
        pass
    
    
    Y_data = pd.DataFrame(np.vstack(Y.values()),columns=Para.Y_labels)
    Y_data.to_csv('data/rawdata/'+runtype+'/Y_ce_ar1_taxes_data.csv')
    y_data = {}
    for t in range(len(y)):
        print("T= " + str(t))
        y_data[t] = pd.DataFrame(y[t],columns=Para.y_labels)
        y_data[t].to_csv('data/rawdata/'+runtype+'/y_ce_ar1_taxes_data_'+str(t)+'.csv')
        
        
    if runtype == "appendix_figure_4":
        
        Y_data.to_csv('data/'+runtype+'.csv')
    
    elif runtype == "appendix_figure_26":

    
        #Now do analysis
        keep_cols=['w_e','n','e','lwe','ly','fi','de']
        y_data=[y_data[t][keep_cols] for t in range(len(y_data)) ]
        
        
        def labelrecessions(lY,stdlY,cutoff=1.):
            T = len(lY)
            recessions = {}
            i = 0
            inrecession = False
            for t in range(T):
                if inrecession:
                    if lY[t] < -stdlY*cutoff:
                        recessions[i].append(t)
                    else:
                        inrecession = False
                        i = i +1
                else:
                    if lY[t] < -stdlY*cutoff:
                        recessions[i] = [t]
                        inrecession = True
            
            return recessions
        
        def computeEarningsLosses(recessions,y_data,Ncut,var='log_we',Tlag=2):
            Nrec = len(recessions)
            losses = []
            for i in range(Nrec):
                recession = recessions[i]
                trec = recession[0]
                if trec-Tlag >=0:
                    logwe_avg = sum([np.exp(y_data[t][var]) for t in range(max(trec-Tlag,0),trec)])
                    logwe_diff = y_data[recession[-1]][var] - y_data[trec-1][var] #total income losses over recession
        
                    #next compute percentiles
                    logwe_perc = pd.qcut(logwe_avg,Ncut,range(Ncut))
                    logwe_dif_perc = [logwe_diff[logwe_perc==i].mean() for i in range(Ncut)]
        
                    losses.append(np.array(logwe_dif_perc))
                
            diff = 100./Ncut
            perc = np.arange(Ncut)*diff + diff/2
            return perc,losses
        Y_data['lY'] = np.log(Y_data.Y) - np.mean(np.log(Y_data.Y))
        Y_data['gr_Y'] = Y_data.lY-Y_data.lY.shift(1)
        std_grY = Y_data.gr_Y.std()
        recessions = labelrecessions(Y_data.gr_Y,std_grY,1.)
        
        Ncut = 21
        perc,losses = computeEarningsLosses(recessions,y_data,Ncut,'ly',10)
        Nrec = len(losses)
        avglosses = sum([losses[i] for i in range(Nrec)])/Nrec
        
        #Now save data
        df1 = pd.DataFrame({'perc':perc,'losses':avglosses/avglosses[int(Ncut/2)]})
        df1.to_csv('data/'+runtype+'_model.csv')
        df2 = pd.DataFrame({'perc':recession_data.index.values,'losses':recession_data.avg.values})
        df2.to_csv('data/'+runtype+'_data.csv') 