# -*- coding: utf-8 -*-
import numpy as np
import pandas as pd
import cPickle as pickle
import simulate_MPI as simulate
import mkl
import getpass
mkl.set_num_threads(1)
import sys



import static_case_revision as static_case


############################
# Load inputs              #
############################

calibrationfile='calibrations/calibration_data/baseline_calibration_revision.csv'
initialconditionsfile='calibrations/calibration_data/init_data_scf_nohtm.csv'

############################
# Load calibration         #
############################

Para = static_case.dotdict()
calibration=pd.read_csv(calibrationfile).T
calibration.columns=calibration.iloc[0,:]
calibration=calibration.iloc[1:,:]
Para.ra = calibration.ra.val
Para.varphi=calibration.varphi.val
Para.gamma =calibration.gamma.val
Para.g = calibration.Gov.val
Para.alpha_0=calibration.alpha_0.val
Para.alpha_1=calibration.alpha_1.val
Para.epsilon =calibration.epsilonbar.val
Para.beta = calibration.beta.val
Para.Tau_w = 0.
Para.Tau_d = 0.#0.34#1.155#
Para.Tau_b = 0.#1.18#
Para.ra = 3.


############################
# Load initial conditions  #
############################
y_data=pd.read_csv(initialconditionsfile)
logw=np.log(y_data.w_e.values)
s=y_data.s_i.values
b=y_data.b.values*0 #no debt so we don't have effect on returs to debt
s = s/s.mean()
logw -= logw.mean()
avgW = ((Para.epsilon-1.)/Para.epsilon ) * np.exp(logw).mean() #compute average wage in model
b = b*avgW #debt was measured as per average wage
Bbar = b.mean()

#regular    
ER = static_case.EquilibriumResiduals(Para,logw,s,b)
z = static_case.computeEqulibrium(ER,np.ones(2))
l = np.exp(ER.interpolate_logl(z))

#Iterate on this until fixed point in shares
w = np.exp(logw)
W = (Para.epsilon-1.)/Para.epsilon
y = np.exp(logw)*l*W
Y = y.mean()
s_l = ((1-Para.Tau_w)*y + Para.Tau_w*Y)/Y

s = s_l





#compute consumption
D = (1-W)*Y
w = np.exp(logw)
c = (l**Para.gamma/((1-Para.Tau_w)*W*w))**(-1./Para.ra)
#Taxes are zero so one impimentation will be pareto weights that are inverse to marginal utilty
omega = 1/c**(-Para.ra)
omega = omega /omega.mean()
mu = 0.*omega
T = 0.
#
############################
# Save results             #
############################

if static_case.rank == 0:
    filename = "data_revision_nohtm_aligned"
    print ("saving initial conditions to filename:")
    print (filename)
    #bencmark
    initial_conditions=logw,Para.g,Para.ra,Para.gamma,Para.beta,c,l,b,omega,T,Para.Tau_w,Para.Tau_d,Para.Tau_b,s,Para.epsilon,mu
    
    pickle.dump( initial_conditions, open( "InitialConditions/"+filename+".pickle", "wb" ) )

