# -*- coding: utf-8 -*-
import numpy as np
import pandas as pd
import cPickle as pickle
import mkl
mkl.set_num_threads(1)
import sys

runtype = "benchmark"


import static_case_revision_htm as static_case


############################
# Load inputs              #
############################

calibrationfile='calibrations/calibration_data/baseline_calibration_revision.csv'
initialconditionsfile='calibrations/calibration_data/init_data_scf_htm.csv'

############################
# Load calibration         #
############################

Para = static_case.dotdict()
calibration=pd.read_csv(calibrationfile).T
calibration.columns=calibration.iloc[0,:]
calibration=calibration.iloc[1:,:]
Para.ra = calibration.ra.val
Para.varphi=calibration.varphi.val
Para.gamma =calibration.gamma.val
Para.g = calibration.Gov.val
Para.alpha_0=calibration.alpha_0.val
Para.alpha_1=calibration.alpha_1.val
Para.epsilon =calibration.epsilonbar.val
Para.beta = calibration.beta.val
Para.Tau_w = 0.38
Para.Tau_d = 0.34
Para.Tau_b = 0.
Para.ra = 3.


############################
# Load initial conditions  #
############################
y_data=pd.read_csv(initialconditionsfile)
logw=np.log(y_data.w_e.values)
s=y_data.s_i.values
b=y_data.b.values
htm=y_data.htm.values
s = s/s.mean()
logw -= logw.mean()
avgW = ((Para.epsilon-1.)/Para.epsilon ) * np.exp(logw).mean() #compute average wage in model
b = b*avgW
Bbar = b.mean()

argv = sys.argv
if len(argv) > 2:
    extension = "_"+argv[1]
else:
    extension = ""

if extension == "":
    True
elif extension == "_poorcih":
    htm = (y_data.cih <= np.percentile(y_data.cih,15)).values 
    b[htm>0.5] = 0.
    b[htm<0.5] -= (b.mean()-Bbar)/(htm<0.5).mean()
else:
    print("Do not recognize extension")
    exit(1)

z,x,c,l,b,omega,T,Tau_w,Tau_d,Tau_b,Y,mu,YSS = static_case.optimal_policy(Para,logw,s,b,htm)
T = z[0]

#
############################
# Save results             #
############################

if static_case.rank == 0:
    filename = "data_revision_htm"+extension
    print ("saving initial conditions to filename:")
    print (filename)
    initial_conditions=logw,Para.g,Para.ra,Para.gamma,Para.beta,c,l,b,omega,T,Tau_w,Tau_d,Tau_b,s,htm,Para.epsilon,mu
    pickle.dump( initial_conditions, open( "InitialConditions/"+filename+".pickle", "wb" ) )

